(function() {
  var buildQuery, validator;

  module.exports = function(schema, options) {
    var message;
    message = 'Expected `{PATH}` to be unique. Value: `{VALUE}`';
    return schema.eachPath(function(path, schemaType) {
      var uniqueness, _ref, _ref1;
      if ((((_ref = schemaType.options) != null ? (_ref1 = _ref.validators) != null ? _ref1.unique : void 0 : void 0) != null)) {
        uniqueness = schemaType.options.validators.unique;
        if (uniqueness === true) {
          schemaType.validators.push({
            validator: validator(path),
            message: message,
            type: 'unique'
          });
        }
        if (typeof uniqueness === "object" && (uniqueness.scope != null)) {
          schemaType.validators.push({
            validator: validator(path, uniqueness.scope),
            message: message,
            type: 'unique'
          });
        }
        if (typeof uniqueness === "object" && (uniqueness.condition != null)) {
          return schemaType.validators.push({
            validator: validator(path, uniqueness.condition),
            message: message,
            type: 'unique'
          });
        }
      }
    });
  };

  buildQuery = function(path, value, id) {
    var query, target;
    target = {};
    target[path] = value;
    query = {
      $and: []
    };
    query.$and.push({
      _id: {
        $ne: id
      }
    });
    query.$and.push(target);
    return query;
  };

  validator = function(path, scopeOrCondition) {
    return function(value, respond) {
      var model, q, query;
      model = this.model(this.constructor.modelName);
      query = buildQuery(path, value, this._id);
      if ((scopeOrCondition != null)) {
        q = {};
        if (typeof scopeOrCondition === "string" || scopeOrCondition.constructor.name === "String") {
          q[scopeOrCondition] = this[scopeOrCondition];
        } else {
          q = scopeOrCondition;
        }
        query.$and.push(q);
      }
      return model.findOne(query, function(err, document) {
        return respond((document != null) === false);
      });
    };
  };

}).call(this);
